# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

try:
    basestring
except NameError:
    basestring = str

from cryptography.fernet import Fernet, InvalidToken

import base64
import hashlib

from django.conf import settings
from django.db.models import Model
from django.http import Http404
from django.shortcuts import get_object_or_404 as go4


__version__ = "1.1.0"
__license__ = "BSD"
__author__ = "Sean Meyer"
__email__ = "slinkymabyday@gmail.com"
__url__ = "https://github.com/slinkymanbyday/django-encrypted-id-cryptography"
__source__ = "https://github.com/slinkymanbyday/django-encrypted-id-cryptography"
__docformat__ = "html"


class EncryptedIDDecodeError(Exception):
    def __init__(self, msg="Failed to decrypt, invalid input."):
        super(EncryptedIDDecodeError, self).__init__(msg)


def encode(the_id):
    assert 0 <= the_id < 2 ** 64
    the_id = str(the_id).encode()
    f = Fernet(settings.ID_ENCRYPT_KEY)
    return f.encrypt(the_id).decode()


def decode(e):
    if e is None:
        raise ValueError()
    if isinstance(e, basestring):
        e = bytes(e.encode("ascii"))

    for skey in getattr(settings, "ID_ENCRYPT_KEYS", [settings.ID_ENCRYPT_KEY]):
        try:
            f = Fernet(skey)
            the_id = f.decrypt(e)
            return int(the_id)
        except ValueError:
            raise EncryptedIDDecodeError("Failed to decrypt, invalid input.")
        except InvalidToken:
            raise EncryptedIDDecodeError("Failed to decrypt, bad decryption key.")

    raise EncryptedIDDecodeError("Failed to decrypt.")


def get_object_or_404(m, ekey, *arg, **kw):
    try:
        pk = decode(ekey)
    except (EncryptedIDDecodeError, ValueError):
        raise Http404

    return go4(m, id=pk, *arg, **kw)


def ekey(instance):
    assert isinstance(instance, Model)
    return encode(instance.pk)



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import os
from setuptools import setup


try:
    long_description = open(b'README.rst', 'rt').read()
except IOError:
    long_description = ""

try:
    long_description += open(b'ChangeLog.rst', 'rt').read().strip()
except IOError:
    pass


MODULE_PATH = os.path.join(os.getcwd(), "encrypted_id", "__init__.py")


def find_this(search, filename=MODULE_PATH):
    """Take a string and a filename path string and return the found value."""
    if not search:
        return
    for line in open(str(filename)).readlines():
        if search.lower() in line.lower():
            line = line.split("=")[1].strip()
            if "'" in line or '"' in line or '"""' in line:
                line = line.replace("'", "").replace('"', '').replace('"""', '')
            return line


print(find_this("__version__"))

setup(
    name="django-encrypted-id-cryptography",
    description="Encrypted IDs for Django Models",
    long_description=long_description,

    version=find_this("__version__"),

    author=find_this("__author__"),
    author_email=find_this("__email__"),
    maintainer=find_this("__author__"),
    maintainer_email=find_this("__email__"),

    url=find_this("__source__"),
    license=find_this("__license__"),


    install_requires=[
        "Django>=1.11", "cryptography",
    ],


    packages=["encrypted_id"],
    zip_safe=True,


    keywords=['Django', 'Web'],


    classifiers=[

        'Development Status :: 5 - Production/Stable',

        'Intended Audience :: Developers',
        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: System Administrators',
        'Intended Audience :: Information Technology',
        'Intended Audience :: Other Audience',

        'Natural Language :: English',

        'Operating System :: OS Independent',
        'Operating System :: POSIX :: Linux',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: MacOS :: MacOS X',

        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.7',

        'Programming Language :: Python :: Implementation :: CPython',

        'Topic :: Software Development',

    ],
)



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from django.db import models
from django.db.models import Lookup, options

from encrypted_id import EncryptedIDDecodeError
from encrypted_id import encode, decode, get_object_or_404

options.DEFAULT_NAMES += ('ek_key',)


@models.ForeignKey.register_lookup
class EkeyLookup(Lookup):
    lookup_name = 'ekey'

    def __init__(self, lhs, rhs):
        try:
            rhs = decode(rhs)
        except EncryptedIDDecodeError:
            raise lhs.target.model.DoesNotExist(
                '%s matching query does not exist.' %
                lhs.target.model._meta.object_name
            )
        super(EkeyLookup, self).__init__(lhs, rhs)

    def as_sql(self, compiler, connection):
        lhs, lhs_params = self.process_lhs(compiler, connection)
        rhs, rhs_params = self.process_rhs(compiler, connection)
        params = lhs_params + rhs_params
        return '%s = %s' % (lhs, rhs), params


class EncryptedIDManager(models.Manager):
    def get_by_ekey(self, ekey, **kw):
        return self.get(pk=decode(ekey), **kw)

    def get_by_ekey_or_404(self, *args, **kw):
        return get_object_or_404(self.model, *args, **kw)


class EncryptedIDQuerySet(models.QuerySet):
    def filter(self, *args, **kw):
        if 'ekey' in kw:
            ekey = kw.pop('ekey')
            try:
                assert ekey is not None
                kw['id'] = decode(ekey)
            except (AssertionError, EncryptedIDDecodeError):
                return self.none()
        return super(EncryptedIDQuerySet, self).filter(*args, **kw)


class EncryptedIDModel(models.Model):
    class Meta:
        abstract = True

    objects = EncryptedIDManager.from_queryset(EncryptedIDQuerySet)()

    @property
    def ekey(self):
        return encode(self.pk)
